@@ -25,7 +25,8 @@
 
 CircuitPython pure-python framebuf module, based on the micropython framebuf module.
 
-* Author(s): Kattni Rembor, Tony DiCola, original file created by Damien P. George
+* Author(s): Melissa LeBlanc-Williams, Kattni Rembor, Tony DiCola, original file
+             created by Damien P. George
 
 Implementation Notes
 --------------------
@@ -52,6 +53,7 @@ MVLSB = 0  # Single bit displays (like SSD1306 OLED)
 RGB565 = 1  # 16-bit color displays
 GS4_HMSB = 2  # Unimplemented!
 MHMSB = 3  # Single bit displays like the Sharp Memory
+RGB888 = 4  # Neopixels and Dotstars
 
 
 class MHMSBFormat:
@@ -142,6 +144,46 @@ class MVLSBFormat:
             height -= 1
 
 
+class RGB888Format:
+    """RGB888Format"""
+
+    @staticmethod
+    def set_pixel(framebuf, x, y, color):
+        """Set a given pixel to a color."""
+        index = (y * framebuf.stride + x) * 3
+        framebuf.buf[index : index + 3] = bytes(
+            ((color >> 16) & 255, (color >> 8) & 255, color & 255)
+        )
+
+    @staticmethod
+    def get_pixel(framebuf, x, y):
+        """Get the color of a given pixel"""
+        index = (y * framebuf.stride + x) * 3
+        return (
+            (framebuf.buf[index] << 16)
+            | (framebuf.buf[index + 1] << 8)
+            | framebuf.buf[index + 2]
+        )
+
+    @staticmethod
+    def fill(framebuf, color):
+        """completely fill/clear the buffer with a color"""
+        fill = (color >> 16) & 255, (color >> 8) & 255, color & 255
+        for i in range(0, len(framebuf.buf), 3):
+            framebuf.buf[i : i + 3] = bytes(fill)
+
+    @staticmethod
+    def fill_rect(framebuf, x, y, width, height, color):
+        """Draw a rectangle at the given location, size and color. The ``fill_rect`` method draws
+        both the outline and interior."""
+        # pylint: disable=too-many-arguments
+        fill = (color >> 16) & 255, (color >> 8) & 255, color & 255
+        for _x in range(x, x + width):
+            for _y in range(y, y + height):
+                index = (_y * framebuf.stride + _x) * 3
+                framebuf.buf[index : index + 3] = bytes(fill)
+
+
 class FrameBuffer:
     """FrameBuffer object.
 
@@ -174,6 +216,8 @@ class FrameBuffer:
             self.format = MVLSBFormat()
         elif buf_format == MHMSB:
             self.format = MHMSBFormat()
+        elif buf_format == RGB888:
+            self.format = RGB888Format()
         else:
             raise ValueError("invalid format")
         self._rotation = 0
@@ -360,25 +404,36 @@ class FrameBuffer:
             if not self._font or self._font.font_name != font_name:
                 # load the font!
                 self._font = BitmapFont(font_name)
-            w = self._font.font_width
+            width = self._font.font_width
+            height = self._font.font_height
             for i, char in enumerate(chunk):
-                self._font.draw_char(
-                    char, x + (i * (w + 1)) * size, y, self, color, size=size
-                )
-            y += self._font.font_height * size
+                char_x = x + (i * (width + 1)) * size
+                if (
+                    char_x + (width * size) > 0
+                    and char_x < self.width
+                    and y + (height * size) > 0
+                    and y < self.height
+                ):
+                    self._font.draw_char(char, char_x, y, self, color, size=size)
+            y += height * size
 
     # pylint: enable=too-many-arguments
 
     def image(self, img):
         """Set buffer to value of Python Imaging Library image.  The image should
         be in 1 bit mode and a size equal to the display size."""
+        # determine our effective width/height, taking rotation into account
+        width = self.width
+        height = self.height
+        if self.rotation == 1 or self.rotation == 3:
+            width, height = height, width
         if img.mode != "1":
             raise ValueError("Image must be in mode 1.")
         imwidth, imheight = img.size
-        if imwidth != self.width or imheight != self.height:
+        if imwidth != width or imheight != height:
             raise ValueError(
                 "Image must be same dimensions as display ({0}x{1}).".format(
-                    self.width, self.height
+                    width, height
                 )
             )
         # Grab all the pixels from the image, faster than getpixel.
@@ -387,8 +442,8 @@ class FrameBuffer:
         for i in range(len(self.buf)):
             self.buf[i] = 0
         # Iterate through the pixels
-        for x in range(self.width):  # yes this double loop is slow,
-            for y in range(self.height):  #  but these displays are small!
+        for x in range(width):  # yes this double loop is slow,
+            for y in range(height):  #  but these displays are small!
                 if pixels[(x, y)]:
                     self.pixel(x, y, 1)  # only write if pixel is true
 
